/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.xmllib.tests;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.InputStream;

import org.junit.BeforeClass; //import org.junit.Ignore;
import org.junit.Ignore;
import org.junit.Test;

import uk.ljmu.metadata.utilities.MetadataUtilities;
import uk.ljmu.qudexapps.xmllib.exceptions.CSVException;
import uk.ljmu.qudexapps.xmllib.services.CsvManagementImpl;

public class CsvManagementTest {
    private static InputStream isCsvFile;
    private static CsvManagementImpl csvImp;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	ClassLoader classLoader = Thread.currentThread()
		.getContextClassLoader();

	csvImp = new CsvManagementImpl();
	// AMG - Init the variables needed
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/csv");
	filePath.append(fileSep);
	filePath.append("csv-general-ss.csv");
	isCsvFile = classLoader.getResourceAsStream(filePath.toString());
	filePath.delete(0, filePath.toString().length());
    }

    // Method readSpreadsheet
    @Test
    public void testReadSpreadsheetNullSS() {
	try {
	    csvImp.readSpreadsheet(null);
	} catch (Exception e) {
	    System.out.println("Null spreadsheet to readSpreadsheet...");
	    assertTrue(e instanceof CSVException);
	}
    }

    @Test
    public void testReadSpreadsheetEmptySS() {
	ClassLoader classLoader = Thread.currentThread()
		.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/csv");
	filePath.append(fileSep);
	filePath.append("empty-ss.csv");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    csvImp.readSpreadsheet(is);
	} catch (Exception e) {
	    System.out.println("Empty spreadsheet to readSpreadsheet...");
	    assertTrue(e instanceof CSVException);
	}
    }

    @Test
    public void testReadSpreadsheetInvalidFormatSS() {
	ClassLoader classLoader = Thread.currentThread()
		.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/csv");
	filePath.append(fileSep);
	filePath.append("invalid-format.xlsx");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    csvImp.readSpreadsheet(is);
	} catch (Exception e) {
	    System.out.println("Invalid format ss to readSpreadsheet...");
	    assertTrue(e instanceof CSVException);
	}
    }

    @Test
    public void testReadSpreadsheetOK() {
	try {
	    boolean res = csvImp.readSpreadsheet(isCsvFile);
	    assertTrue(res);
	} catch (CSVException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Empty spreadsheet to readSpreadsheet...");
	    fail(e.getMessage());
	}
    }

    // Method constructDocument
    @Test
    public void testConstructDocumentOKFoxml() {
	// Very important this test just applies to NON Qudex SS
	csvImp.setQudexSS(false);
	// Construct spreadsheet
	ClassLoader classLoader = Thread.currentThread()
		.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/csv");
	filePath.append(fileSep);
	filePath.append("csv-general-ss.csv");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = csvImp.readSpreadsheet(is);
	    if (res) {
		String data = csvImp.constructDocument("demo:testcsv1",
			MetadataUtilities.ValueType.DcElement, "demo:testcsv1");
		assertTrue(!data.equals(""));
	    } else fail("Errors while reading spreadsheet");
	} catch (CSVException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Empty spreadsheet to readSpreadsheet...");
	    fail(e.getMessage());
	}
    }

    @Test
    public void testConstructDocumentNoRelsDc() {
	// Construct spreadsheet
	ClassLoader classLoader = Thread.currentThread()
		.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/csv");
	filePath.append(fileSep);
	filePath.append("csv-norelsdc-ss.csv");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = csvImp.readSpreadsheet(is);
	    if (res) {
		String data = csvImp.constructDocument("demo:test1",
			MetadataUtilities.ValueType.DcElement, "demo:test1");
		String data2 = csvImp.constructDocument("demo:test1",
			MetadataUtilities.ValueType.RELS_EXT, "demo:test1");
		
		assertTrue(data.equals("") && data2.equals(""));		
	    } else fail("Errors while reading spreadsheet");
	} catch (CSVException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Empty spreadsheet to readSpreadsheet...");
	    fail(e.getMessage());
	}
    }

    @Test
    public void testConstructDocumentRDF() {
	// Construct spreadsheet
	ClassLoader classLoader = Thread.currentThread()
		.getContextClassLoader();
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/csv");
	filePath.append(fileSep);
	filePath.append("csv-rdf-ss.csv");
	InputStream is = classLoader.getResourceAsStream(filePath.toString());
	try {
	    boolean res = csvImp.readSpreadsheet(is);
	    if (res) {
		String data = csvImp.constructDocument("demo:test1",
			MetadataUtilities.ValueType.RELS_EXT, "demo:test1");
		assertTrue(!data.equals(""));
	    } else fail("Errors while reading spreadsheet");
	} catch (CSVException e) {
	    System.out.println("Test failure, readSpreadsheet threw exception");
	    System.out.println(e.getMessage());
	    fail(e.getMessage());
	} catch (IOException e) {
	    System.out.println("Empty spreadsheet to readSpreadsheet...");
	    fail(e.getMessage());
	}
    }
    //TODO Implement this test
    @Ignore
    public void testFindSchemaOK() {

    }
}
