/*
 * Copyright 2013 Agustina Martinez-Garcia

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
package uk.ljmu.qudexapps.xmllib.tests;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.xmlbeans.XmlError;
import org.apache.xmlbeans.XmlOptions;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.junit.BeforeClass;
import org.junit.AfterClass;
import org.junit.Test; 
// import org.junit.Ignore;

import fedora.fedoraSystemDef.foxml.DigitalObjectDocument.DigitalObject;

import static org.junit.Assert.*;

import uk.ljmu.metadata.schemasutilities.SchemasProcessing;
import uk.ljmu.metadata.utilities.MetadataUtilities;
import uk.ljmu.qudexapps.xmllib.exceptions.FoxmlException;
import uk.ljmu.qudexapps.xmllib.model.FoxmlBeans;
import uk.ljmu.qudexapps.xmllib.services.QudexFoxmlService;
import uk.ljmu.qudexapps.xmllib.services.QudexFoxmlServiceImpl;
import uk.ljmu.qudexapps.xmllib.utilities.Constants;
import uk.ljmu.qudexapps.xmllib.utilities.IngestionResults;

public class QudexFoxmlServiceTest {
    private static String testIdentifier = "qudex:collection1";
    private static QudexFoxmlService qudexServiceCsv;
    private static QudexFoxmlService qudexServiceExcel;
    private static InputStream isCsvFile;
    private static InputStream isExcelFile;
    private static InputStream isCsvCollection;
    private static InputStream isExcelCollection;
    private static List<String> ingestedTestObjects;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
	ClassLoader classLoader = Thread.currentThread()
	.getContextClassLoader();
	// AMG - Init the variables needed
	qudexServiceCsv = new QudexFoxmlServiceImpl(
		MetadataUtilities.MIMETYPES_MAP.get("csv"), true);
	qudexServiceExcel = new QudexFoxmlServiceImpl(
		MetadataUtilities.MIMETYPES_MAP.get("excel"), true);
	String fileSep = System.getProperty("file.separator", "/");
	StringBuilder filePath = new StringBuilder();
	filePath.append("test-spreadsheets/qudex");
	filePath.append(fileSep);
	filePath.append("test-qudex-documents.xls");
	isExcelFile = classLoader.getResourceAsStream(filePath.toString());
	filePath.delete(0, filePath.toString().length());
	filePath.append("test-spreadsheets/qudex");
	filePath.append(fileSep);
	filePath.append("test-qudex-collections.xls");
	isExcelCollection = classLoader
	.getResourceAsStream(filePath.toString());
	filePath.delete(0, filePath.toString().length());
	filePath.append("test-spreadsheets/qudex");
	filePath.append(fileSep);
	filePath.append("test-qudex-documents.csv");
	isCsvFile = classLoader.getResourceAsStream(filePath.toString());
	filePath.delete(0, filePath.toString().length());
	filePath.append("test-spreadsheets/qudex");
	filePath.append(fileSep);
	filePath.append("test-qudex-collections.csv");
	isCsvCollection = classLoader.getResourceAsStream(filePath.toString());
	ingestedTestObjects = new ArrayList<String>();
	createTestCollection();
    }

    @AfterClass
    public static void setUpAfterClass() throws Exception {
	removeTestObjects();
	if (isCsvCollection != null)
	    isCsvCollection.close();
	if (isCsvFile != null)
	    isCsvFile.close();
	if (isExcelCollection != null)
	    isExcelCollection.close();
	if (isExcelCollection != null)
	    isExcelCollection.close();
    }

    // TEST Environment
    // Collection-level spreadsheets
    @Test
    public void testIngestCollectionNullSSExcel() {

	try {
	    qudexServiceExcel.ingestQudexCollectionsSpreadsheet(null, null);
	} catch (Exception e) {
	    assertTrue(e instanceof FoxmlException);
	} finally {
	    qudexServiceExcel.getObjResults().clear();
	}
    }

    @Test
    public void testIngestCollectionNullSSCsv() {

	try {
	    qudexServiceCsv.ingestQudexCollectionsSpreadsheet(null, null);
	} catch (Exception e) {
	    assertTrue(e instanceof FoxmlException);
	} finally {
	    qudexServiceCsv.getObjResults().clear();
	}
    }

    @Test
    public void testIngestCollectionSSCsv() {
	int objsIngested = 0;
	try {
	    qudexServiceCsv.ingestQudexCollectionsSpreadsheet(new ArrayList<String>(), isCsvCollection);
	    List<IngestionResults> results = qudexServiceCsv.getObjResults();
	    for (IngestionResults res : results) {
		if (res.getGeneratedObjectPID().startsWith("qudex")) {
		    ingestedTestObjects.add(res.getGeneratedObjectPID());
		    objsIngested++;
		}
	    }	    
	    assertTrue(objsIngested == results.size());
	    qudexServiceCsv.getObjResults().clear();
	} catch (FoxmlException e) {
	    fail(e.getMessage());
	}
    }

    @Test
    public void testIngestCollectionSSExcel() {
	int objsIngested = 0;
	try {
	    qudexServiceExcel.ingestQudexCollectionsSpreadsheet(new ArrayList<String>(), isExcelCollection);
	    List<IngestionResults> results = qudexServiceExcel.getObjResults();
	    for (IngestionResults res : results) {
		if (res.getGeneratedObjectPID().startsWith("qudex")) {
		    ingestedTestObjects.add(res.getGeneratedObjectPID());
		    objsIngested++;
		}		    
	    }	    
	    assertTrue(results.size() == objsIngested);
	    qudexServiceExcel.getObjResults().clear();
	} catch (FoxmlException e) {
	    fail(e.getMessage());
	}
    }

    @Test
    public void testIngestFileSSCsv() {
	int objsIngested = 0;
	try {
	    List<String> colsIds = new ArrayList<String>();
	    colsIds.add(testIdentifier);
	    qudexServiceCsv.setQudexCollections(false);
	    qudexServiceCsv.ingestFileLevelSpreadsheet(colsIds, isCsvFile);
	    List<IngestionResults> results = qudexServiceCsv.getObjResults();
	    for (IngestionResults res : results) {
		if (res.getGeneratedObjectPID().startsWith("qudex")) {
		    ingestedTestObjects.add(res.getGeneratedObjectPID());
		    objsIngested++;
		}
	    }	    
	    assertTrue(results.size() == objsIngested);
	    qudexServiceCsv.getObjResults().clear();
	} catch (FoxmlException e) {
	    fail(e.getMessage());
	}
    }

    @Test
    public void testIngestFileSSExcel() {
	int objsIngested = 0;
	try {
	    List<String> colsIds = new ArrayList<String>();
	    colsIds.add(testIdentifier);
	    qudexServiceExcel.setQudexCollections(false);
	    qudexServiceExcel.ingestFileLevelSpreadsheet(colsIds, isExcelFile);
	    List<IngestionResults> results = qudexServiceExcel.getObjResults();
	    for (IngestionResults res : results) {
		if (res.getGeneratedObjectPID().startsWith("qudex")) {
		    ingestedTestObjects.add(res.getGeneratedObjectPID());
		    objsIngested++;
		}
	    }	    
	    assertTrue(results.size() == objsIngested);
	    qudexServiceExcel.getObjResults().clear();
	} catch (FoxmlException e) {
	    fail(e.getMessage());
	}
    }

    private static void removeTestObjects() {
	for (String pid : ingestedTestObjects) {
	    qudexServiceExcel.getFedoraObject().deleteObject(pid);
	}
    }

    private static boolean createTestCollection() {
	// AMG - Check that the collection doesn't exist
	if (!qudexServiceExcel.getFedoraObject().getObjectXML(testIdentifier)
		.contains("xml")) {
	    FoxmlBeans foxmlBeans = new FoxmlBeans(qudexServiceExcel
		    .getFedoraObject());
	    // Create object, it does not exist
	    StringBuilder sb = new StringBuilder();

	    boolean ok = createFoxmlFileTest(foxmlBeans, "qudex", "true");
	    if (ok) {
		// RELS_EXT RDF-XML CONTENTS FOR THE DS
		Document doc = null;

		// Construct RELS-EXT document
		List<String> prefixList = new ArrayList<String>();
		prefixList.add("rel");
		prefixList.add("rdf");
		prefixList.add("dcmitype");
		doc = SchemasProcessing.contructRDFDocument(foxmlBeans
			.getFoxmlDoc().getDigitalObject().getPID(), prefixList);

		String type = MetadataUtilities.DCMI_ELEMENTS.get("collection");
		SchemasProcessing.addRDFProperty("rdf", "rdf:type", type,
			MetadataUtilities.propertyTypes.Resource.toString(), doc);

		try {
		    foxmlBeans.addFedoraRelsExtDatastream(doc.asXML(),
			    Constants.DEFAULT_MIMETYPE);
		} catch (FoxmlException ex) {
		}

		// VALIDATE FOXML INSTANCE THROUGH SCHEMA
		try {
		    // CALL VALIDATION FUNCTION
		    // Create an XmlOptions instance and set the error listener.
		    XmlOptions validateOptions = new XmlOptions();
		    ArrayList<XmlError> errorList = new ArrayList<XmlError>();
		    validateOptions.setErrorListener(errorList);

		    // FOXML DOCUMENT VALIDATION
		    boolean validDocument = foxmlBeans.getFoxmlDoc().validate(
			    validateOptions);
		    // If the XML isn't valid, loop through the listener's
		    // contents,
		    // printing contained messages.
		    if (!validDocument) {
			for (int i = 0; i < errorList.size(); i++) {
			    XmlError error = (XmlError) errorList.get(i);
			    sb.append("RESULTS OF VALIDATION \nMessage: ");
			    sb.append(error.getMessage());
			    sb.append("Location of invalid XML: ");
			    sb.append(error.getCursorLocation().xmlText());
			    sb.delete(0, sb.toString().length());
			}
			System.out.println(sb.toString());
		    } else {
			// CALL TO INGEST METHOD
			String pid = qudexServiceExcel.getQuObject().ingestObjectFedora("Test Object",
				"qudex", foxmlBeans.getFoxmlDoc().toString(),
				foxmlBeans.getFoxmlDoc().getDigitalObject()
				.getPID());
			ingestedTestObjects.add(pid);
			testIdentifier = pid;
		    }
		} catch (Exception e) {
		    System.out.println(e.getMessage());
		}
	    }
	}
	return true;
    }

    private static boolean createFoxmlFileTest(FoxmlBeans foxmlBeans,
	    String namespace, String xmlFormat) {
	String pid = "";
	String pidListXML = "";
	DigitalObject digObj;

	// AMG- Obtain current Foxml Digital Object instance
	digObj = foxmlBeans.getFoxmlDoc().getDigitalObject();
	foxmlBeans
	.addLabelProperty("Test qudex collection, QudexService tests.");
	foxmlBeans.addOwnerIDProperty("fedoraAdmin");
	foxmlBeans.addStateProperty("A");
	// AMg- Parameters: number of PIDs, optional namespace associate to the
	// PID, XML output format
	pidListXML = qudexServiceExcel.getFedoraObject().getNextPID(1,
		namespace, xmlFormat);

	if (pidListXML == null || pidListXML == "") {
	    return false;
	}

	// AMG- Parse XML response to obtain the generated PID
	try {
	    Document doc = DocumentHelper.parseText(pidListXML);
	    Element pidElem = doc.getRootElement().element("pid");
	    pid = pidElem.getText();
	    digObj.setPID(pid);
	} catch (DocumentException ex) {
	    return false;
	}

	return true;
    }
}